#include <iostream>
#include <vector>
using namespace std;

const int Dim = 26;
int n, q, op, x, y, res;
char ch;
string s;

class SegTree {
public:
    SegTree() {}
    SegTree(const int& n) : n {n}
	{
        arb = vector<int>(4 * n);
        Build(1, 1, n);
    }
    
    void Update(const int& pos, const int& val) 
    {
        Update(1, 1, n, pos, val);
    }
    
    int Query(const int& st, const int& dr) 
    {
        return Query(1, 1, n, st, dr);
    }
private:
    void Build(int node, int st, int dr) 
    {
        if (st == dr) 
        {
            arb[node] = 1 << (s[st - 1] - 'a'); // 2^(s[st - 1] - 'a')
            return;
        }
        int mid = (st + dr) / 2;
        Build(2 * node, st, mid);
        Build(2 * node + 1, mid + 1, dr);
        arb[node] = (arb[2 * node] | arb[2 * node + 1]);
    }
    
    void Update(int node, int st, int dr, const int& pos, const int& val) 
    {
        if (st == dr) 
        {
            arb[node] = val;
            return;
        }
        int mid = (st + dr) / 2;
        if (pos <= mid)
            Update(2 * node, st, mid, pos, val);
        else Update(2 * node + 1, mid + 1, dr, pos, val);
        
        arb[node] = (arb[2 * node] | arb[2 * node + 1]);
    }
    
    int Query(int node, int st, int dr, const int& l, const int& r) 
    {
        if (l <= st && dr <= r)
            return arb[node];
        int mid = (st + dr) / 2, r1(0), r2(0);
        if (l <= mid)
            r1 = Query(2 * node, st, mid, l, r);
        if (mid < r)
            r2 = Query(2 * node + 1, mid + 1, dr, l, r);
        return (r1 | r2);
    }
private:
    int n;
    vector<int> arb;
};

int main() 
{
    ios_base::sync_with_stdio(false);
    cin.tie(0), cout.tie(0);
    
    cin >> n >> s >> q;
    SegTree st(n);
    for (int i = 1; i <= q; ++i) 
    {
        cin >> op >> x;
        if (op == 1) {
            cin >> ch;
            st.Update(x, 1 << (ch - 'a'));
        }
        else {
            cin >> y;
            cout << __builtin_popcount(st.Query(x, y)) << '\n';
        }
    }
    return 0;
}
